#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-


################################################
#############   MACHINE LEARNING   #############
##############     -  2016 -    ################
################################################

"""
Contact: Adrian Ulges (adrian.ulges@hs-rm.de)
"""

import pylab
import matplotlib.pyplot as plt
from scipy.stats import multivariate_normal
import numpy as np


def plot(X, model=None):
    """
    this function plots a set of 2D points (X)
    and (optionally) a color map corresponding to the
    the density function resulting from an EM clustering (model).

    @type X: numpy.array
    @param X: the data to plot. shape must be (n,2) (where
              n is the number of points to plot).
    @type model: pair(numpy.array, numpy.array)
    @param model: a pair (means,vars), where 
              - means is a (K,2)-Array containing the K cluster centers
              - vars  is a (K,2)-Array containing the K cluster's variances in each dimension. 
    """
    # set the plot's range based on the data's minimum and maximum
    # along the x-/y-axis.
    [xmin,xmax] = [ X[:,0].min(), X[:,0].max() ]
    [ymin,ymax] = [ X[:,1].min(), X[:,1].max() ]
    
    # extend the range a bit at the top+bottom end
    [xmin,xmax] = [xmin-0.1*(xmax-xmin), xmax+0.1*(xmax-xmin)]
    [ymin,ymax] = [ymin-0.1*(ymax-ymin), ymax+0.1*(ymax-ymin)]

    # plot the samples
    plt.plot(X[:,0], X[:,1], 'o', color='gray', markersize=3, linewidth=4)
    plt.xlim( [xmin,xmax] )
    plt.ylim( [ymin,ymax] )

    # if model is specified, add a color plot
    if model is not None:

        (means,vars) = model
        K = means.shape[0]

        # define a 20x20 grid of positions
        x = np.arange(xmin, xmax, (xmax-xmin)/20.)
        y = np.arange(ymin, ymax, (ymax-ymin)/20.)
        F = np.zeros((len(x), len(y)))

        # at each position, compute the density function
        for i,xval in enumerate(x):
            for j,yval in enumerate(y):
                for m,v in zip(means,vars):
                    F[i,j] += 1./K * multivariate_normal.pdf([xval,yval], 
                                                             mean=m, cov=np.diag(v))

        # color plot
        im = plt.imshow(F.T, interpolation='bicubic', origin='lower',
                    cmap=pylab.get_cmap("RdYlGn"), extent=[xmin,xmax,ymin,ymax] )

    plt.grid()

    plt.show()



def generate_test_data(nsamples=100):
    """
    this function generates a sample dataset of 2D points
    that belong to 3 multivariate Gaussian clusters.
    The output of this function can be plotted with plot()
    (see above).

    @type X: numpy.array
    @param X: the data to plot. shape must be (n,2) (where
              n is the number of points to plot).
    @rtype: numpy.array
    @returns: an 300 x 2 data matrix (3 clusters, 100 points per cluster,
              all 2-dimensional)
    """

    # one cluster with center (2,4)
    mu0 = [2,4]
    sigma0 = np.array([[0.5,0.8],[0.8,5]])
    samples0 = np.random.multivariate_normal(mu0,sigma0,nsamples).T

    # one cluster with center (7,2)
    mu1 = [7,2]
    sigma1 = np.array([[2,0.0],[0.,0.7]])
    samples1 = np.random.multivariate_normal(mu1,sigma1,nsamples).T

    # one cluster with center (7,6)
    mu2 = [7,6]
    sigma2 = np.array([[0.5,-0.3],[-0.3,0.7]])
    samples2 = np.random.multivariate_normal(mu2,sigma2,nsamples).T

    # merge samples from 3 clusters together
    return np.hstack([samples0,samples1,samples2]).T
